from __future__ import annotations

from typing import Any, Collection, Iterator, Literal

import numpy.typing as npt

from ..h5g import GroupID
from .attrs import AttributeManager
from .base import HLObject, MutableMappingHDF5
from .dataset import Dataset
from .datatype import Datatype

class Group(HLObject, MutableMappingHDF5[str, "Group" | Dataset | Datatype]):
    def __init__(self, bind: GroupID): ...
    def __delitem__(self, name: str) -> None: ...
    def require_group(self, name: str) -> Group: ...
    def __getitem__(
        self, name: str | slice | tuple[()]
    ) -> Group | Dataset | Datatype: ...
    def __setitem__(self, name: str, obj: Any) -> None: ...
    def __iter__(self) -> Iterator[str]: ...
    def __len__(self) -> int: ...
    @property
    def attrs(self) -> AttributeManager: ...
    def create_group(self, name: str, track_order: bool | None = None) -> Group: ...
    def create_dataset(
        self,
        name: str | None,
        shape: int | tuple[()] | tuple[int | None, ...] | None = None,
        dtype: npt.DTypeLike | None = None,
        data: Collection[Any] | None = None,
        **kwds: Any,
    ) -> Dataset: ...
    @property
    def id(self) -> GroupID: ...
    def move(self, source: str, dest: str) -> None: ...
    @property
    def mode(self) -> Literal["r", "r+"]: ...
    @property
    def parent(self) -> Group: ...



import numpy as np

from typing import Any
from typing import Literal

class string_info(tuple[str, int]): ...

def string_dtype(
    encoding: Literal["utf-8", "ascii"] = "utf-8", length: int | None = None
) -> np.dtype[np.bytes_] | np.dtype[np.object_]: ...
def check_string_dtype(*args: Any, **kwargs: Any) -> string_info | None: ...



import numpy as np
from numpy.typing import NDArray

from typing import Any
from typing import Iterator

from . import base
from .base import Empty

class AttributeManager(
    base.MutableMappingHDF5[str, Empty | NDArray[np.generic] | np.generic],
    base.CommonStateObject,
):
    def __delitem__(self, name: str) -> None: ...
    def __getitem__(self, name: str) -> Empty | NDArray[np.generic] | np.generic: ...
    def __setitem__(self, name: str, value: Any) -> None: ...
    def __iter__(self) -> Iterator[str]: ...
    def __len__(self) -> int: ...



from ._hl.group import Group
from ._hl.files import File
from ._hl.dataset import Dataset
from ._hl.datatype import Datatype
from ._hl.base import HLObject
from .h5t import string_dtype

__all__ = ["Group", "File", "Dataset", "Datatype", "HLObject", "string_dtype"]



from typing import Any

from .group import Group

class File(Group):
    def close(self) -> None: ...
    def __enter__(self) -> File: ...
    def __exit__(self, *args: Any) -> None: ...
    @property
    def filename(self) -> str: ...
    def flush(self) -> None: ...



from abc import ABCMeta
from collections.abc import Mapping
from collections.abc import KeysView
from collections.abc import ItemsView
from collections.abc import ValuesView
from collections.abc import MutableMapping

from typing import TypeVar

from .._objects import ObjectID
from .files import File
from .attrs import AttributeManager

_K = TypeVar("_K", bound=str)
_V = TypeVar("_V")

class KeysViewHDF5(KeysView[_K]): ...
class ValuesViewHDF5(ValuesView[_V]): ...
class ItemsViewHDF5(ItemsView[_K, _V]): ...
class CommonStateObject: ...

class HLObject(CommonStateObject):
    @property
    def id(self) -> ObjectID: ...
    @property
    def name(self) -> str: ...
    @property
    def file(self) -> File: ...
    @property
    def attrs(self) -> AttributeManager: ...

class MappingHDF5(Mapping[_K, _V], metaclass=ABCMeta):
    def keys(self) -> KeysViewHDF5[_K]: ...
    def values(self) -> ValuesViewHDF5[_V]: ...
    def items(self) -> ItemsViewHDF5[_K, _V]: ...
    def _ipython_key_completions_(self) -> KeysViewHDF5[_K]: ...

class MutableMappingHDF5(
    MappingHDF5[_K, _V], MutableMapping[_K, _V], metaclass=ABCMeta
): ...
class Empty: ...



from numbers import Number
from types import EllipsisType
from typing import Any, Collection, Iterable, Literal, SupportsIndex, Union, overload

import numpy as np
import numpy.typing as npt
from numpy._typing import _ArrayLikeInt_co

from ..h5d import DatasetID
from .base import HLObject

SELECTOR = Union[
    None,
    EllipsisType,
    tuple[()],
    int,
    bool,
    SupportsIndex,
    slice,
    range,
    Iterable[int],
    Iterable[bool],
    _ArrayLikeInt_co,
]

class AstypeWrapper:
    """Wrapper to convert data on reading from a dataset."""

    def __init__(self, dset: Dataset, dtype: np.generic): ...
    def __getitem__(self, args: tuple[int, ...]) -> npt.NDArray[Any] | np.generic: ...
    def __enter__(self) -> AstypeWrapper: ...
    def __exit__(self, *args: None) -> None: ...
    def __len__(self) -> int: ...

class AsStrWrapper:
    """Wrapper to decode strings on reading the dataset"""

    def __init__(
        self,
        dset: Dataset,
        encoding: Literal["ascii", "utf-8"],
        errors: Literal["strict"] = "strict",
    ): ...
    def __getitem__(self, args: tuple[int, ...]) -> npt.NDArray[np.object_]: ...
    def __len__(self) -> int: ...

class ChunkIterator:
    def __iter__(self) -> ChunkIterator: ...
    def __next__(self) -> tuple[slice, ...]: ...

class Dataset(HLObject):
    def __init__(self, bind: DatasetID, *, readonly: bool = False): ...
    @overload
    def resize(self, size: int, axis: int) -> None: ...
    @overload
    def resize(self, size: Collection[int], axis: None = None) -> None: ...
    def __getitem__(
        self,
        args: SELECTOR | tuple[SELECTOR, ...],
        new_dtype: npt.DTypeLike | None = None,
    ) -> Any: ...
    def __setitem__(
        self,
        args: SELECTOR | tuple[SELECTOR, ...],
        val: npt.NDArray[Any] | Number | str,
    ) -> None: ...
    @property
    def ndim(self) -> int: ...
    @property
    def id(self) -> DatasetID: ...
    @property
    def dtype(self) -> np.dtype[np.generic]: ...
    @property
    def chunks(self) -> tuple[int, ...] | None: ...
    @property
    def shape(self) -> tuple[int, ...]: ...
    @property
    def size(self) -> int: ...
    def astype(self, dtype: npt.DTypeLike) -> AstypeWrapper: ...
    def asstr(
        self,
        encoding: Literal["ascii", "utf-8"] | None = None,
        errors: Literal["strict"] = "strict",
    ) -> AsStrWrapper: ...
    def __len__(self) -> int: ...
    def iter_chunks(
        self, sel: slice | tuple[slice, ...] | None = None
    ) -> ChunkIterator: ...
    def read_direct(
        self,
        dest: npt.NDArray[Any],
        source_sel: tuple[int | slice | Collection[int], ...] | None = None,
        dest_sel: tuple[int | slice | Collection[int], ...] | None = None,
    ) -> None: ...
    def write_direct(
        self,
        source: npt.NDArray[Any],
        source_sel: tuple[int | slice | Collection[int] | None, ...] | None = None,
        dest_sel: tuple[int | slice | Collection[int] | None, ...] | None = None,
    ) -> None: ...
